/*
    SPDX-FileCopyrightText: 2002 Frerich Raabe <raabe@kde.org>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "treebuilder.h"

using namespace KHC;

TreeBuilder::TreeBuilder(QObject *parent)
    : QObject(parent)
{
}

void TreeBuilder::selectURL(const QString &)
{
}

#include "moc_treebuilder.cpp"

// vim:ts=2:sw=2:et
