/****************************************************************************
** Meta object code from reading C++ file 'searchhandler.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../khelpcenter-26.07.80/searchhandler.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'searchhandler.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3KHC9SearchJobE_t {};
} // unnamed namespace

template <> constexpr inline auto KHC::SearchJob::qt_create_metaobjectdata<qt_meta_tag_ZN3KHC9SearchJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KHC::SearchJob",
        "searchFinished",
        "",
        "SearchJob*",
        "DocEntry*",
        "searchError",
        "searchExited",
        "exitCode",
        "QProcess::ExitStatus",
        "slotJobResult",
        "KJob*",
        "job",
        "slotJobData",
        "KIO::Job*",
        "data"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'searchFinished'
        QtMocHelpers::SignalData<void(SearchJob *, DocEntry *, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 2 }, { QMetaType::QString, 2 },
        }}),
        // Signal 'searchError'
        QtMocHelpers::SignalData<void(SearchJob *, DocEntry *, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 2 }, { QMetaType::QString, 2 },
        }}),
        // Slot 'searchExited'
        QtMocHelpers::SlotData<void(int, QProcess::ExitStatus)>(6, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { 0x80000000 | 8, 2 },
        }}),
        // Slot 'slotJobResult'
        QtMocHelpers::SlotData<void(KJob *)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'slotJobData'
        QtMocHelpers::SlotData<void(KIO::Job *, const QByteArray &)>(12, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 13, 2 }, { QMetaType::QByteArray, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SearchJob, qt_meta_tag_ZN3KHC9SearchJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KHC::SearchJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KHC9SearchJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KHC9SearchJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3KHC9SearchJobE_t>.metaTypes,
    nullptr
} };

void KHC::SearchJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SearchJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->searchFinished((*reinterpret_cast<std::add_pointer_t<SearchJob*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DocEntry*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 1: _t->searchError((*reinterpret_cast<std::add_pointer_t<SearchJob*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DocEntry*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 2: _t->searchExited((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QProcess::ExitStatus>>(_a[2]))); break;
        case 3: _t->slotJobResult((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 4: _t->slotJobData((*reinterpret_cast<std::add_pointer_t<KIO::Job*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SearchJob* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SearchJob* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SearchJob::*)(SearchJob * , DocEntry * , const QString & )>(_a, &SearchJob::searchFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SearchJob::*)(SearchJob * , DocEntry * , const QString & )>(_a, &SearchJob::searchError, 1))
            return;
    }
}

const QMetaObject *KHC::SearchJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KHC::SearchJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KHC9SearchJobE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KHC::SearchJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void KHC::SearchJob::searchFinished(SearchJob * _t1, DocEntry * _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void KHC::SearchJob::searchError(SearchJob * _t1, DocEntry * _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}
namespace {
struct qt_meta_tag_ZN3KHC13SearchHandlerE_t {};
} // unnamed namespace

template <> constexpr inline auto KHC::SearchHandler::qt_create_metaobjectdata<qt_meta_tag_ZN3KHC13SearchHandlerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KHC::SearchHandler",
        "searchFinished",
        "",
        "SearchHandler*",
        "DocEntry*",
        "searchError"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'searchFinished'
        QtMocHelpers::SignalData<void(SearchHandler *, DocEntry *, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 2 }, { QMetaType::QString, 2 },
        }}),
        // Signal 'searchError'
        QtMocHelpers::SignalData<void(SearchHandler *, DocEntry *, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 2 }, { QMetaType::QString, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SearchHandler, qt_meta_tag_ZN3KHC13SearchHandlerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KHC::SearchHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KHC13SearchHandlerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KHC13SearchHandlerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3KHC13SearchHandlerE_t>.metaTypes,
    nullptr
} };

void KHC::SearchHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SearchHandler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->searchFinished((*reinterpret_cast<std::add_pointer_t<SearchHandler*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DocEntry*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 1: _t->searchError((*reinterpret_cast<std::add_pointer_t<SearchHandler*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DocEntry*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SearchHandler* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SearchHandler* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SearchHandler::*)(SearchHandler * , DocEntry * , const QString & )>(_a, &SearchHandler::searchFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SearchHandler::*)(SearchHandler * , DocEntry * , const QString & )>(_a, &SearchHandler::searchError, 1))
            return;
    }
}

const QMetaObject *KHC::SearchHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KHC::SearchHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KHC13SearchHandlerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KHC::SearchHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void KHC::SearchHandler::searchFinished(SearchHandler * _t1, DocEntry * _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void KHC::SearchHandler::searchError(SearchHandler * _t1, DocEntry * _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}
namespace {
struct qt_meta_tag_ZN3KHC28ExternalProcessSearchHandlerE_t {};
} // unnamed namespace

template <> constexpr inline auto KHC::ExternalProcessSearchHandler::qt_create_metaobjectdata<qt_meta_tag_ZN3KHC28ExternalProcessSearchHandlerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KHC::ExternalProcessSearchHandler",
        "slotSearchFinished",
        "",
        "SearchJob*",
        "DocEntry*",
        "slotSearchError"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotSearchFinished'
        QtMocHelpers::SlotData<void(SearchJob *, DocEntry *, const QString &)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 2 }, { QMetaType::QString, 2 },
        }}),
        // Slot 'slotSearchError'
        QtMocHelpers::SlotData<void(SearchJob *, DocEntry *, const QString &)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 2 }, { QMetaType::QString, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ExternalProcessSearchHandler, qt_meta_tag_ZN3KHC28ExternalProcessSearchHandlerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KHC::ExternalProcessSearchHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<SearchHandler::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KHC28ExternalProcessSearchHandlerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KHC28ExternalProcessSearchHandlerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3KHC28ExternalProcessSearchHandlerE_t>.metaTypes,
    nullptr
} };

void KHC::ExternalProcessSearchHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ExternalProcessSearchHandler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotSearchFinished((*reinterpret_cast<std::add_pointer_t<SearchJob*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DocEntry*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 1: _t->slotSearchError((*reinterpret_cast<std::add_pointer_t<SearchJob*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DocEntry*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SearchJob* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SearchJob* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *KHC::ExternalProcessSearchHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KHC::ExternalProcessSearchHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KHC28ExternalProcessSearchHandlerE_t>.strings))
        return static_cast<void*>(this);
    return SearchHandler::qt_metacast(_clname);
}

int KHC::ExternalProcessSearchHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SearchHandler::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
