/********************************************************************************
** Form generated from reading UI file 'pagesearchbar.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PAGESEARCHBAR_H
#define UI_PAGESEARCHBAR_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace KHC {

class Ui_PageSearchBar
{
public:
    QHBoxLayout *hboxLayout;
    QLineEdit *searchTextEdit;
    QToolButton *nextButton;
    QToolButton *previousButton;
    QToolButton *matchCaseCheckButton;
    QToolButton *hideButton;

    void setupUi(QWidget *KHC__PageSearchBar)
    {
        if (KHC__PageSearchBar->objectName().isEmpty())
            KHC__PageSearchBar->setObjectName("KHC__PageSearchBar");
        KHC__PageSearchBar->resize(582, 35);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(KHC__PageSearchBar->sizePolicy().hasHeightForWidth());
        KHC__PageSearchBar->setSizePolicy(sizePolicy);
        hboxLayout = new QHBoxLayout(KHC__PageSearchBar);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        searchTextEdit = new QLineEdit(KHC__PageSearchBar);
        searchTextEdit->setObjectName("searchTextEdit");
        searchTextEdit->setClearButtonEnabled(true);

        hboxLayout->addWidget(searchTextEdit);

        nextButton = new QToolButton(KHC__PageSearchBar);
        nextButton->setObjectName("nextButton");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("go-down-search")));
        nextButton->setIcon(icon);
        nextButton->setAutoRaise(true);

        hboxLayout->addWidget(nextButton);

        previousButton = new QToolButton(KHC__PageSearchBar);
        previousButton->setObjectName("previousButton");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("go-up-search")));
        previousButton->setIcon(icon1);
        previousButton->setAutoRaise(true);

        hboxLayout->addWidget(previousButton);

        matchCaseCheckButton = new QToolButton(KHC__PageSearchBar);
        matchCaseCheckButton->setObjectName("matchCaseCheckButton");
        QIcon icon2(QIcon::fromTheme(QString::fromUtf8("format-text-superscript")));
        matchCaseCheckButton->setIcon(icon2);
        matchCaseCheckButton->setCheckable(true);
        matchCaseCheckButton->setAutoRaise(true);

        hboxLayout->addWidget(matchCaseCheckButton);

        hideButton = new QToolButton(KHC__PageSearchBar);
        hideButton->setObjectName("hideButton");
        QIcon icon3(QIcon::fromTheme(QString::fromUtf8("dialog-close")));
        hideButton->setIcon(icon3);
        hideButton->setAutoRaise(true);

        hboxLayout->addWidget(hideButton);


        retranslateUi(KHC__PageSearchBar);

        QMetaObject::connectSlotsByName(KHC__PageSearchBar);
    } // setupUi

    void retranslateUi(QWidget *KHC__PageSearchBar)
    {
        searchTextEdit->setPlaceholderText(tr2i18n("Search...", "@info:placeholder"));
#if QT_CONFIG(tooltip)
        nextButton->setToolTip(tr2i18n("Jump to next match", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        previousButton->setToolTip(tr2i18n("Jump to previous match", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        matchCaseCheckButton->setToolTip(tr2i18n("Match case sensitive", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)KHC__PageSearchBar;
    } // retranslateUi

};

} // namespace KHC

namespace KHC {
namespace Ui {
    class PageSearchBar: public Ui_PageSearchBar {};
} // namespace Ui
} // namespace KHC

#endif // PAGESEARCHBAR_H

