/****************************************************************************
** Meta object code from reading C++ file 'navigator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../khelpcenter-26.07.80/navigator.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'navigator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3KHC9NavigatorE_t {};
} // unnamed namespace

template <> constexpr inline auto KHC::Navigator::qt_create_metaobjectdata<qt_meta_tag_ZN3KHC9NavigatorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KHC::Navigator",
        "itemSelected",
        "",
        "itemURL",
        "glossSelected",
        "GlossaryEntry",
        "entry",
        "setStatusBarText",
        "text",
        "openInternalUrl",
        "QUrl",
        "url",
        "slotItemSelected",
        "QTreeWidgetItem*",
        "index",
        "slotItemExpanded",
        "item",
        "slotItemCollapsed",
        "slotSearch",
        "slotShowSearchResult",
        "slotSelectGlossEntry",
        "id",
        "selectItem",
        "slotSearchFinished",
        "checkSearchEdit",
        "clearSearch",
        "slotDelayedIndexingStart",
        "slotDoIndexWork",
        "slotProcessExited",
        "exitCode",
        "QProcess::ExitStatus",
        "exitStatus",
        "slotShowIndexingProgressBar"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'itemSelected'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'glossSelected'
        QtMocHelpers::SignalData<void(const GlossaryEntry &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'setStatusBarText'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'openInternalUrl'
        QtMocHelpers::SlotData<void(const QUrl &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'slotItemSelected'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'slotItemExpanded'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 16 },
        }}),
        // Slot 'slotItemCollapsed'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 16 },
        }}),
        // Slot 'slotSearch'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowSearchResult'
        QtMocHelpers::SlotData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotSelectGlossEntry'
        QtMocHelpers::SlotData<void(const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Slot 'selectItem'
        QtMocHelpers::SlotData<void(const QUrl &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'slotSearchFinished'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'checkSearchEdit'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'clearSearch'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDelayedIndexingStart'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDoIndexWork'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotProcessExited'
        QtMocHelpers::SlotData<void(int, QProcess::ExitStatus)>(28, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 29 }, { 0x80000000 | 30, 31 },
        }}),
        // Slot 'slotShowIndexingProgressBar'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Navigator, qt_meta_tag_ZN3KHC9NavigatorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KHC::Navigator::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KHC9NavigatorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KHC9NavigatorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3KHC9NavigatorE_t>.metaTypes,
    nullptr
} };

void KHC::Navigator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Navigator *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->itemSelected((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->glossSelected((*reinterpret_cast<std::add_pointer_t<GlossaryEntry>>(_a[1]))); break;
        case 2: _t->setStatusBarText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->openInternalUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 4: _t->slotItemSelected((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1]))); break;
        case 5: _t->slotItemExpanded((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1]))); break;
        case 6: _t->slotItemCollapsed((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1]))); break;
        case 7: _t->slotSearch(); break;
        case 8: _t->slotShowSearchResult((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->slotSelectGlossEntry((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->selectItem((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 11: _t->slotSearchFinished(); break;
        case 12: _t->checkSearchEdit(); break;
        case 13: _t->clearSearch(); break;
        case 14: _t->slotDelayedIndexingStart(); break;
        case 15: _t->slotDoIndexWork(); break;
        case 16: _t->slotProcessExited((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QProcess::ExitStatus>>(_a[2]))); break;
        case 17: _t->slotShowIndexingProgressBar(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Navigator::*)(const QString & )>(_a, &Navigator::itemSelected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Navigator::*)(const GlossaryEntry & )>(_a, &Navigator::glossSelected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Navigator::*)(const QString & )>(_a, &Navigator::setStatusBarText, 2))
            return;
    }
}

const QMetaObject *KHC::Navigator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KHC::Navigator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KHC9NavigatorE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int KHC::Navigator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void KHC::Navigator::itemSelected(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KHC::Navigator::glossSelected(const GlossaryEntry & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KHC::Navigator::setStatusBarText(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
