/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../khelpcenter-26.07.80/mainwindow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3KHC10MainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto KHC::MainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN3KHC10MainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KHC::MainWindow",
        "D-Bus Interface",
        "org.kde.khelpcenter.khelpcenter",
        "openUrl",
        "",
        "url",
        "startup_id",
        "showHome",
        "lastSearch",
        "statusBarRichTextMessage",
        "m",
        "statusBarMessage",
        "slotShowHome",
        "slotLastSearch",
        "slotFullScreen",
        "showSearchStderr",
        "viewUrl",
        "QUrl",
        "enableLastSearchAction",
        "enableCopyTextAction",
        "slotPrintPreview",
        "slotPrint",
        "slotGlossSelected",
        "GlossaryEntry",
        "entry",
        "slotStarted",
        "slotInfoMessage",
        "KJob*",
        "goInternalUrl",
        "slotOpenURLRequest",
        "documentCompleted",
        "slotCopySelectedText"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'openUrl'
        QtMocHelpers::SlotData<void(const QString &)>(3, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'openUrl'
        QtMocHelpers::SlotData<void(const QString &, const QByteArray &)>(3, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { QMetaType::QByteArray, 6 },
        }}),
        // Slot 'showHome'
        QtMocHelpers::SlotData<void()>(7, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'lastSearch'
        QtMocHelpers::SlotData<void()>(8, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'statusBarRichTextMessage'
        QtMocHelpers::SlotData<void(const QString &)>(9, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'statusBarMessage'
        QtMocHelpers::SlotData<void(const QString &)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'slotShowHome'
        QtMocHelpers::SlotData<void()>(12, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotLastSearch'
        QtMocHelpers::SlotData<void()>(13, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFullScreen'
        QtMocHelpers::SlotData<void()>(14, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showSearchStderr'
        QtMocHelpers::SlotData<void()>(15, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewUrl'
        QtMocHelpers::SlotData<void(const QString &)>(16, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'openUrl'
        QtMocHelpers::SlotData<void(const QUrl &)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 5 },
        }}),
        // Slot 'enableLastSearchAction'
        QtMocHelpers::SlotData<void()>(18, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'enableCopyTextAction'
        QtMocHelpers::SlotData<void()>(19, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotPrintPreview'
        QtMocHelpers::SlotData<void()>(20, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotPrint'
        QtMocHelpers::SlotData<void()>(21, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotGlossSelected'
        QtMocHelpers::SlotData<void(const GlossaryEntry &)>(22, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'slotStarted'
        QtMocHelpers::SlotData<void()>(25, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotInfoMessage'
        QtMocHelpers::SlotData<void(KJob *, const QString &)>(26, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 27, 4 }, { QMetaType::QString, 4 },
        }}),
        // Slot 'goInternalUrl'
        QtMocHelpers::SlotData<void(const QUrl &)>(28, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 4 },
        }}),
        // Slot 'slotOpenURLRequest'
        QtMocHelpers::SlotData<void(const QUrl &)>(29, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 5 },
        }}),
        // Slot 'documentCompleted'
        QtMocHelpers::SlotData<void()>(30, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCopySelectedText'
        QtMocHelpers::SlotData<void()>(31, 4, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<MainWindow, qt_meta_tag_ZN3KHC10MainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject KHC::MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<KXmlGuiWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KHC10MainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KHC10MainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3KHC10MainWindowE_t>.metaTypes,
    nullptr
} };

void KHC::MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->openUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->openUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 2: _t->showHome(); break;
        case 3: _t->lastSearch(); break;
        case 4: _t->statusBarRichTextMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->statusBarMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->slotShowHome(); break;
        case 7: _t->slotLastSearch(); break;
        case 8: _t->slotFullScreen(); break;
        case 9: _t->showSearchStderr(); break;
        case 10: _t->viewUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->openUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 12: _t->enableLastSearchAction(); break;
        case 13: _t->enableCopyTextAction(); break;
        case 14: _t->slotPrintPreview(); break;
        case 15: _t->slotPrint(); break;
        case 16: _t->slotGlossSelected((*reinterpret_cast<std::add_pointer_t<GlossaryEntry>>(_a[1]))); break;
        case 17: _t->slotStarted(); break;
        case 18: _t->slotInfoMessage((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 19: _t->goInternalUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 20: _t->slotOpenURLRequest((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 21: _t->documentCompleted(); break;
        case 22: _t->slotCopySelectedText(); break;
        default: ;
        }
    }
}

const QMetaObject *KHC::MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KHC::MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3KHC10MainWindowE_t>.strings))
        return static_cast<void*>(this);
    return KXmlGuiWindow::qt_metacast(_clname);
}

int KHC::MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KXmlGuiWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 23;
    }
    return _id;
}
QT_WARNING_POP
